/*global define*/
/*jslint vars:true, browser:true*/
define([
    'testsuite/qunit',
    'testsuite/qunit-fixture!de_epages/ebay/ui/ebayassignmentcontroller-test.html',
    'jquery',
    'de_epages/ebay/ui/ebayassignmentcontroller',
    'ep/core',
    "$dict!de_epages/ebay/dictionary",

    '$ready!'
], function (QUnit, fixture, $, de_epages, ep, ebayDictionary) {
    'use strict';

    /*
     * @dictionary      ep.dict
     *
     * @translation     {Apply}
     *                  {Error}
     *                  {INVALID_LENGTH}
     *                  {MsgInvalidProductCode}
     *                  {UnknownError}
     *
     * @dictionary      de_epages.ebay.dictionary
     *
     * @translation     {noShopEnvironment}
     *                  {EbayRequestFailed}
     *                  {EbayItems}
     *                  {EbayAttributes}
     *                  {NoEbayItemsFound}
     *                  {UnityConnectionNotSaved}
     *                  {PleaseConsultYourProvider}
     */

    // translation object
    var translations = {
            EbayWarningNoEbayEnvironment: ebayDictionary.translate('noShopEnvironment'),
            NoEbayEnvironment: ebayDictionary.translate('noShopEnvironment'),
            EbayItems: ebayDictionary.translate('EbayItems'),
            EbayAttributes: ebayDictionary.translate('EbayAttributes'),
            NoEbayItemsFound: ebayDictionary.translate('NoEbayItemsFound'),
            UnityConnectionNotSaved: ebayDictionary.translate('UnityConnectionNotSaved'),
            PleaseConsultYourProvider: ebayDictionary.translate('PleaseConsultYourProvider'),

            error: ebayDictionary.translate('Error'),
            unknownError: ebayDictionary.translate('UnknownError'),
            apply: ebayDictionary.translate('Apply'),
            INVALID_LENGTH: ebayDictionary.translate('INVALID_LENGTH'),
            MsgInvalidProductCode: ebayDictionary.translate('MsgInvalidProductCode')

        },
        errors = {
            "ApiCallError": {
                header: translations.error,
                description: translations.UnityConnectionNotSaved + " " + translations.PleaseConsultYourProvider
            },
            "DefaultError": {
                header: translations.error,
                description: translations.unknownError
            }
        };

    var products = [{
                    "epid": "219540209",
                    "gtin": [
                        "0885609001920"
                    ],
                    "brand": "Dyson",
                    "mpn": [
                        "204720-01"
                    ],
                    "image": {
                        "imageUrl": "https://i.ebayimg.com/00/s/NTY4WDEwMDA=/z/TDEAAOSwJQdW-mXz/$_6.JPG?set_id=89040003C1"
                    },
                    "aspects": [{
                            "localizedName": "Brand",
                            "localizedValues": [
                                "Dyson"
                            ]
                        },
                        {
                            "localizedName": "Model",
                            "localizedValues": [
                                "DC58"
                            ]
                        },
                        {
                            "localizedName": "MPN",
                            "localizedValues": [
                                "20472001"
                            ]
                        },
                        {
                            "localizedName": "Type",
                            "localizedValues": [
                                "Handheld"
                            ]
                        },
                        {
                            "localizedName": "Filters",
                            "localizedValues": [
                                "HEPA"
                            ]
                        }
                    ],
                    "ean": [
                        "0885609001920"
                    ],
                    "upc": [
                        "0885609001920"
                    ],
                    "productHref": "https://api.sandbox.ebay.com/commerce/catalog/v1_beta/product/219540209",
                    "title": "Dyson DC58 - Purple/Silver - Handheld Cleaner",
                    "productWebUrl": "https://www.ebay.com/p/Dyson-DC58-Purple-Silver-Handheld-Cleaner/219540209?src=urllib"
                },
                {
                    "epid": "1232143",
                    "gtin": ["088"],
                    "brand": "Knoedelschleuder",
                    "mpn": ["204720-01"],
                    "image": {
                        "imageUrl": "https://i.ebayimg.com/00/s/NTY4WDEwMDA=/z/TDEAAOSwJQdW-mXz/$_6.JPG?set_id=89040003C1"
                    },
                    "aspects": [{
                        "localizedName": "Brand",
                        "localizedValues": ["Knoedelschleuder"]
                    }, {
                        "localizedName": "Type",
                        "localizedValues": ["Nahrungergaenzugsmittel"]
                    }],
                    "ean": ["0885609001920"],
                    "upc": ["0885609001920"],
                    "productHref": "https://api.sandbox.ebay.com/commerce/catalog/v1_beta/product/219540209",
                    "title": "tolle Knoedelschleuder",
                    "productWebUrl": "https://www.ebay.com/p/Dyson-DC58-Purple-Silver-Handheld-Cleaner/219540209?src=urllib"
                }
            ],

        responseData = {
            "success": true,
            "products": JSON.stringify(products)
        },

        emptyResponseData = {
            "success": true,
            "products": JSON.stringify([])
        };

    QUnit.module("de_epages/ebay/ui/ebayassignmentcontroller", {
        setup: fixture.init
        // ,
        // teardown: fixture.destroy
    });


    QUnit.asyncTest("widget test presence of methods", 10, function () {

        $.each([
            '_create',
            '_renderSearchTable',
            '_search',
            '_validate',
            '_renderError',
            '_resetResult',
            'disableSaveButton',
            '_isJQueryElement',
            '_getElement',
            'destroy'
        ], function (i, fn) {
            QUnit.equal(
                $.isFunction($.ui.ebayAssignmentcontroller.prototype[fn]),
                true,
                '$.isFunction(' + fn + '):'
            );
        });

        QUnit.start();
    });

    QUnit.asyncTest("test UI", function () {

        /* START - fake ajax request */
        ep.ajax = function (o) {
            var result = $.Deferred(),
                resolveSend;

            if (o.data && o.data.ChangeAction) {
                if (o.data.ChangeAction === "JSONSearchCatalogProductByGtin") {
                    if (o.data.SearchString === "357030843X") {
                        result.resolve(responseData);
                    } else if(o.data.SearchString === "0400100836590") {
                        result.resolve({
                            success: false,
                            error: 'ApiCallError'
                        });
                    } else {
                        result.resolve(emptyResponseData);
                    }
                }
            } else {
                result.reject();
            }

            return result;
        };
        /* END - fake ajax request */
        var assignmentController = $('.ep-ebay-assignment-controller'),
            searchInput = $('.ep-ebay-assignment-search-input'),
            searchButton = $('.ep-ebay-assignment-search-button'),
            saveButton = $('.ep-ebay-assignment-save-button'),
            errorElem = $('.ep-ebay-assignment-controller-errors'),

            rows,
            radios,
            i;

        // instantiate ebay assignment controller widget
        assignmentController.ebayAssignmentcontroller({
            epid: 219540209,
            searchButton: '.ep-ebay-assignment-search-button',
            searchInput:  '.ep-ebay-assignment-search-input',
            saveButton: '.ep-ebay-assignment-save-button',
            errorElem: '.ep-ebay-assignment-controller-errors',
            objectId: '12345'
        });

        setTimeout(function() {
            QUnit.ok(1, '==== Initial rendering without GTIN =====');
            // check initial state
            QUnit.equal(
                $('.ep-ebay-assignment-controller').find('.ep-ebay-assignment-no-items').length,
                1,
                'Element for "no items message" is shown.'
            );
            QUnit.equal(
                saveButton.prop('disabled'),
                false,
                'Save button is enabled.'
            );

            QUnit.ok(1, '==== GTIN Search with empty result =====');
            // check empty response for products
            searchInput.val("4012345678901");
            searchButton.trigger('click');

            setTimeout(function() {
                QUnit.equal(
                    $('.ep-ebay-assignment-controller').find('.ep-ebay-assignment-no-items').length,
                    1,
                    'Finishing a request without products => element for "no items message" is shown.'
                );
                QUnit.equal(
                    saveButton.prop('disabled'),
                    true,
                    'Save button is disabled.'
                );

                // check response with products
                QUnit.ok(1, '==== GTIN Search with result list/table =====');
                searchInput.val("357030843X");
                searchButton.trigger('click');

                setTimeout(function() {
                    rows = $('.ep-ebay-assignment-response-table-row');
                    QUnit.equal(
                        rows.length,
                        products.length,
                        'Number of result rows should be 2.'
                    );

                    radios = $('.ep-ebay-assignment-item-epid-radio');
                    QUnit.equal(
                        radios.length,
                        products.length,
                        'Number radio buttons should be 2'
                    )

                    for(i=0; i < products.length; i++){
                        QUnit.equal(
                            radios.eq(i).val(),
                            products[i].epid,
                            'epid "'+radios.eq(i).val()+'" of radio button'+(i+1)+'should be the same as the product epid "'+products[i].epid+'"'
                        )
                    }

                    QUnit.equal(
                        saveButton.prop('disabled'),
                        true,
                        'Save button is disabled.'
                    );

                    QUnit.ok(1, '==== Check radio button in result list/table =====');
                    radios.eq(0).trigger('click');

                    setTimeout(function() {
                        QUnit.equal(
                            radios.filter(':checked').length,
                            1,
                            'One radio button is checked.'
                        );

                        QUnit.equal(
                            saveButton.prop('disabled'),
                            false,
                            'Save button is enabled.'
                        );

                        // check GTIN with invalid number of digits and trigger input event
                        QUnit.ok(1, '==== GTIN Search with wrong number of digits/chars =====');
                        searchInput
                            .val("12345")
                            .trigger('input');

                        setTimeout(function() {
                            QUnit.equal(
                                saveButton.prop('disabled'),
                                true,
                                'Change value in search input field => save button is disabled.'
                            );

                            searchButton.trigger('click');

                            setTimeout(function() {
                                QUnit.equal(
                                    $('.ep-uiTooltip div').html(),
                                    translations.INVALID_LENGTH,
                                    'Search for a invalid GTIN with wrong length => should display a tooltip containing "' + translations.INVALID_LENGTH + '"'
                                );

                                // check GTIN with invalid checksum
                                QUnit.ok(1, '==== GTIN Search with wrong checksum =====');
                                searchInput.val("4012345678903");
                                searchButton.trigger('click');

                                setTimeout(function() {
                                    QUnit.equal(
                                        $('.ep-uiTooltip div').html(),
                                        translations.MsgInvalidProductCode,
                                        'Search for a invalid GTIN with wrong checksum => should display a tooltip containing "' + translations.MsgInvalidProductCode + '"'
                                    );

                                    // check response without connection to ebay
                                    QUnit.ok(1, '==== GTIN Search with no connection to ebay =====');
                                    searchInput.val("0400100836590");
                                    searchButton.trigger('click');

                                    setTimeout(function() {
                                        QUnit.equal(
                                            errorElem.find('p').html(),
                                            errors.ApiCallError.description,
                                            'No connection to ebay => should display an error message containing "' + errors.ApiCallError.description + '"'
                                        );

                                        // destroy widget
                                        QUnit.ok(1, '==== Check destroy method of widget =====');
                                        assignmentController.ebayAssignmentcontroller('destroy');
                                        QUnit.equal(
                                            $('.ep-ebay-assignment-response-table').length,
                                            0,
                                            'Destroy method removed html elements.'
                                        );

                                        searchInput.val("357030843X");
                                        // instantiate ebay assignment controller widget
                                        QUnit.ok(1, '==== Initial rendering with valid GTIN =====');
                                        assignmentController.ebayAssignmentcontroller({
                                            epid: products[1].epid,
                                            searchButton: '.ep-ebay-assignment-search-button',
                                            searchInput:  '.ep-ebay-assignment-search-input',
                                            saveButton: '.ep-ebay-assignment-save-button',
                                            errorElem: '.ep-ebay-assignment-controller-errors',
                                            objectId: '12345'
                                        });

                                        setTimeout(function() {
                                            radios = $('.ep-ebay-assignment-item-epid-radio');

                                            QUnit.equal(
                                                radios.eq(1).val(),
                                                products[1].epid,
                                                'Second radio buttons value is: ' + products[1].epid
                                            );

                                            QUnit.ok(
                                                radios.eq(1).prop('checked'),
                                                'Second radio buttons value is checked.'
                                            );

                                            QUnit.start();
                                        }, 100);
                                    }, 100);
                                }, 100);
                            }, 100);
                        }, 100);
                    }, 200);
                }, 100);
            }, 100);
        }, 100);
    });
});